#!/usr/bin/env python
#  
#  For more information, please see: http://software.sci.utah.edu
#  
#  The MIT License
#  
#  Copyright (c) 2011 Scientific Computing and Imaging Institute,
#  University of Utah.
#  
#  
#  Permission is hereby granted, free of charge, to any person obtaining a
#  copy of this software and associated documentation files (the "Software"),
#  to deal in the Software without restriction, including without limitation
#  the rights to use, copy, modify, merge, publish, distribute, sublicense,
#  and/or sell copies of the Software, and to permit persons to whom the
#  Software is furnished to do so, subject to the following conditions:
#  
#  The above copyright notice and this permission notice shall be included
#  in all copies or substantial portions of the Software.
#  
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
#  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
#  THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
#  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
#  DEALINGS IN THE SOFTWARE.
#  

from sys import argv
from sys import exit
import os
import shutil
import subprocess
import sys
import time
import traceback
import glob

# tests for the existence of either a "done" or
# iter.[iter #] file
def test_monitor_condition( output_path ) :
	found = glob.glob( os.path.join( output_path,"done" ) ) + \
			glob.glob( os.path.join( output_path, "iter*" ) )
	if ( found != [] ) :
		return True
	return False

def wait_for_signal( output_path ) :
	while ( not( test_monitor_condition( output_path ) ) ) :
		time.sleep( 2 )

if __name__ == "__main__" :
	if len( argv ) != 3 :
		print( 'Usage: %s working_directory_path stage_id.' % argv[0] )
		exit( 1 )
	if ( not( os.path.isdir( argv[1] ) ) ) :
		print( 'Working directory path is not a directory: %s.' % argv[1] )
		exit( 1 )

	scripts_path = os.path.dirname(argv[0])

	working_dir_path = argv[1]	
	current_stage_id = argv[2]

	# check for the existence of a parameter file in the input directory.
	parameter_fname = os.path.join(working_dir_path,"params.txt")
	if ( not( os.path.exists( parameter_fname ) ) ) :
		print( 'No such file %s.' % parameter_fname )
		exit( 1 )
	
	# source the parameter.txt file to read in the variables
	# note: this is a security issue if the origin of this file is not 
	# trusted.
	try:
		parameter_file = open( parameter_fname ).read()
	except:
		print( 'Could not open %s for reading.' % parameter_fname )  
		exit( 1 )

	# Run the params.txt file so that its variables get read in.
	# If this fails, the following line will return an exception.
	# We choose not to handle it, as the commandline error is more
	# informative this way.
	try:
		exec( parameter_file )
	
	except:
		print( 'Failed to exec %s in monitor.py' % parameter_fname )
		traceback.print_exc()
		exit( 1 )
	
	current_stage_index = 0
	
	# find the index of the current stage.

	valid_stage = False

	try:
		for stage_id in stage_ids :
			if ( stage_id == current_stage_id ) :
				valid_stage = True
				break;
			current_stage_index = current_stage_index + 1
	except NameError:
		print( 'stage_ids undefined\n' )
		exit( 1 )
	
	if ( not( valid_stage ) ) :
		print( 'Invalid stage specified' )
		exit( 1 )
	
	if ( current_stage_index < 1 or current_stage_index > 4 ) :
		print( 'Nothing to run for this stage.  Exiting.\n')
		exit( 0 )

	stage_specific_fname = os.path.join( scripts_path, current_stage_id+'.py' )
	
	try:
		stage_specific = open( stage_specific_fname ).read()
	except:
		print( 'Could not open %s for reading.' % stage_specific_fname )  
		exit( 1 )
	
	try:
		exec( stage_specific )
	except:
		print( 'Failed to execute %s in monitor.py' % stage_specific_fname )
		traceback.print_exc()
		exit( 1 )

	wait_for_signal( working_dir_path )
	
